Class: Dnsruby::Dnssec::ValidationPolicy
- Inherits:
-
Object
- Object
- Dnsruby::Dnssec::ValidationPolicy
- Defined in:
- lib/dnsruby/dnssec.rb
Constant Summary collapse
- ALWAYS_ROOT_ONLY =
- Always use the root and ignore local trust anchors.
1- ROOT_THEN_LOCAL_ANCHORS =
- Use the root if successful, otherwise try local anchors.
2- LOCAL_ANCHORS_THEN_ROOT =
- Use local trust anchors if available, otherwise use root.
3- ALWAYS_LOCAL_ANCHORS_ONLY =
- Always use local trust anchors and ignore the root.
4