Method: Dnsruby::Message#tsigerror

Defined in:
lib/dnsruby/message/message.rb

#tsigerrorObject

If this message has been verified using a TSIG RR then tsigerror contains the error code returned by the TSIG verification. The error will be an RCode



159
160
161
# File 'lib/dnsruby/message/message.rb', line 159

def tsigerror
  @tsigerror
end