Class: Dnsruby::RR::CDNSKEY
- Defined in:
- lib/dnsruby/resource/CDNSKEY.rb
Overview
RFC4034, section 2 DNSSEC uses public key cryptography to sign and authenticate DNS resource record sets (RRsets). The public keys are stored in DNSKEY resource records and are used in the DNSSEC authentication process described in [RFC4035]: A zone signs its authoritative RRsets by using a private key and stores the corresponding public key in a DNSKEY RR. A resolver can then use the public key to validate signatures covering the RRsets in the zone, and thus to authenticate them.
Constant Summary collapse
Constants inherited from DNSKEY
DNSKEY::REVOKED_KEY, DNSKEY::SEP_KEY, DNSKEY::ZONE_KEY
Instance Attribute Summary
Attributes inherited from DNSKEY
#algorithm, #flags, #key, #key_length, #protocol
Method Summary
Methods inherited from DNSKEY
decode_rdata, #dsa_key, #ec_key, #encode_rdata, #from_data, #from_hash, #from_string, #generate_key_tag, #get_new_key_tag, #init_defaults, #key_tag, #key_tag_pre_revoked, #public_key, #rdata_to_string, #revoked=, #revoked?, #rsa_key, #sep_key=, #sep_key?, #zone_key=, #zone_key?