Method: Dnsruby::Resolv.instance
- Defined in:
- lib/dnsruby/resolv.rb
.instance ⇒ Object
Some class methods require the use of an instance to compute their result. For this purpose we create a single instance that can be reused.
20 21 22 |
# File 'lib/dnsruby/resolv.rb', line 20 def self.instance @instance ||= self.new end |