Class: Dnsruby::Resolver

Inherits:
Object
  • Object
show all
Defined in:
lib/Dnsruby/Resolver.rb

Overview

Description

Dnsruby::Resolver is a DNS stub resolver. This class performs queries with retries across multiple nameservers. The system configured resolvers are used by default.

The retry policy is a combination of the Net::DNS and dnsjava approach, and has the option of :

  • A total timeout for the query (defaults to 0, meaning “no total timeout”)

  • A retransmission system that targets the namervers concurrently once the first query round is

complete, but in which the total time per query round is split between the number of nameservers 
targetted for the first round. and total time for query round is doubled for each query round

Note that, if a total timeout is specified, then that will apply regardless of the retry policy (i.e. it may cut retries short).

Note also that these timeouts are distinct from the SingleResolver’s packet_timeout

Timeouts apply to the initial query and response. If DNSSEC validation is to be performed, then additional queries may be required (these are performed automatically by Dnsruby). Each additional query will be performed with its own timeouts. So, even with a query_timeout of 5 seconds, a response which required extensive validation may take several times that long. (Future versions of Dnsruby may expose finer-grained events for client tracking of responses and validation)

Methods

Synchronous

These methods raise an exception or return a response message with rcode==NOERROR

  • Dnsruby::Resolver#send_message(msg)

  • Dnsruby::Resolver#query(name [, type [, klass]])

Asynchronous

These methods use a response queue to return the response and the error

  • Dnsruby::Resolver#send_async(msg, response_queue, query_id)

Event Loop

Dnsruby runs a pure Ruby event loop to handle I/O in a single thread. Support for EventMachine has been deprecated.

Direct Known Subclasses

SingleResolver

Defined Under Namespace

Classes: EventType

Constant Summary collapse

DefaultQueryTimeout =
0
DefaultPacketTimeout =
10
DefaultRetryTimes =
4
DefaultRetryDelay =
5
DefaultPort =
53
DefaultDnssec =
true
AbsoluteMinDnssecUdpSize =
1220
MinDnssecUdpSize =
4096
DefaultUDPSize =
MinDnssecUdpSize

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(*args) ⇒ Resolver

Create a new Resolver object. If no parameters are passed in, then the default system configuration will be used. Otherwise, a Hash may be passed in with the following optional elements :

  • :port

  • :use_tcp

  • :tsig

  • :ignore_truncation

  • :src_address

  • :src_port

  • :recurse

  • :udp_size

  • :config_info - see Config

  • :nameserver - can be either a String or an array of Strings

  • :packet_timeout

  • :query_timeout

  • :retry_times

  • :retry_delay



350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
# File 'lib/Dnsruby/Resolver.rb', line 350

def initialize(*args)
  # @TODO@ Should we allow :namesver to be an RRSet of NS records? Would then need to randomly order them?
  @resolver_ruby = nil
  @src_address = nil
  @single_res_mutex = Mutex.new
  @configured = false
  @config = Config.new()
  reset_attributes
  
  # Process args
  if (args.length==1)
    if (args[0].class == Hash)
      args[0].keys.each do |key|
        begin
          if (key == :config_info)
            @config.set_config_info(args[0][:config_info])
          elsif (key==:nameserver)
            set_config_nameserver(args[0][:nameserver])
          elsif (key==:nameservers)
            set_config_nameserver(args[0][:nameservers])
          else
            send(key.to_s+"=", args[0][key])
          end
        rescue Exception
          Dnsruby.log.error{"Argument #{key} not valid\n"}
        end
      end
    elsif (args[0].class == String)
      set_config_nameserver(args[0])          
    elsif (args[0].class == Config)
      # also accepts a Config object from Dnsruby::Resolv
      @config = args[0]
    end
  else
    # Anything to do?
  end
  #      if (@single_resolvers==[])
  #        add_config_nameservers
  #      end
  update
  #      ResolverRegister::register_resolver(self)
end

Instance Attribute Details

#configObject (readonly)

The current Config



100
101
102
# File 'lib/Dnsruby/Resolver.rb', line 100

def config
  @config
end

#dnssecObject

Use DNSSEC for this Resolver



133
134
135
# File 'lib/Dnsruby/Resolver.rb', line 133

def dnssec
  @dnssec
end

#do_validationObject

Defines whether validation is performed by default on this Resolver when the query method is called. Note that send_message and send_async expect a Message object to be passed in, which is already configured to the callers requirements.



140
141
142
# File 'lib/Dnsruby/Resolver.rb', line 140

def do_validation
  @do_validation
end

#ignore_truncationObject

Should truncation be ignored? i.e. the TC bit is ignored and thus the resolver will not requery over TCP if TC is set



88
89
90
# File 'lib/Dnsruby/Resolver.rb', line 88

def ignore_truncation
  @ignore_truncation
end

#packet_timeoutObject

The timeout for any individual packet. This is the timeout used by SingleResolver



118
119
120
# File 'lib/Dnsruby/Resolver.rb', line 118

def packet_timeout
  @packet_timeout
end

#portObject

The port to send queries to on the resolver



78
79
80
# File 'lib/Dnsruby/Resolver.rb', line 78

def port
  @port
end

#query_timeoutObject

Note that this timeout represents the total time a query may run for - multiple packets can be sent to multiple nameservers in this time. This is distinct from the SingleResolver per-packet timeout The query_timeout is not required - it will default to 0, which means “do not use query_timeout”. If this is the case then the timeout will be dictated by the retry_times and retry_delay attributes



125
126
127
# File 'lib/Dnsruby/Resolver.rb', line 125

def query_timeout
  @query_timeout
end

#recurseObject

Should the Recursion Desired bit be set?



94
95
96
# File 'lib/Dnsruby/Resolver.rb', line 94

def recurse
  @recurse
end

#retry_delayObject

The query will be tried across nameservers retry_times times, with a delay of retry_delay seconds between each retry. The first time round, retry_delay will be divided by the number of nameservers being targetted, and a new nameserver will be queried with the resultant delay.



130
131
132
# File 'lib/Dnsruby/Resolver.rb', line 130

def retry_delay
  @retry_delay
end

#retry_timesObject

The query will be tried across nameservers retry_times times, with a delay of retry_delay seconds between each retry. The first time round, retry_delay will be divided by the number of nameservers being targetted, and a new nameserver will be queried with the resultant delay.



130
131
132
# File 'lib/Dnsruby/Resolver.rb', line 130

def retry_times
  @retry_times
end

#src_addressObject

The source address to send queries from



91
92
93
# File 'lib/Dnsruby/Resolver.rb', line 91

def src_address
  @src_address
end

#tsigObject

Returns the value of attribute tsig.



84
85
86
# File 'lib/Dnsruby/Resolver.rb', line 84

def tsig
  @tsig
end

#udp_sizeObject

The maximum UDP size to be used



97
98
99
# File 'lib/Dnsruby/Resolver.rb', line 97

def udp_size
  @udp_size
end

#use_tcpObject

Should TCP be used as a transport rather than UDP?



81
82
83
# File 'lib/Dnsruby/Resolver.rb', line 81

def use_tcp
  @use_tcp
end

Class Method Details

.check_port(p, src_port = []) ⇒ Object



559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
# File 'lib/Dnsruby/Resolver.rb', line 559

def Resolver.check_port(p, src_port=[])
  if (p.class != Fixnum)
    tmp_src_ports = Array.new(src_port)
    p.each do |x|
      if (!Resolver.check_port(x, tmp_src_ports))
        return false
      end
      tmp_src_ports.push(x)
    end
    return true
  end
  if (Resolver.port_in_range(p))
    if ((p == 0) && (src_port.length > 0))
      return false
    end
    return true
  else
    Dnsruby.log.error("Illegal port (#{p})")
    raise ArgumentError.new("Illegal port #{p}")
  end
end

.get_ports_from(p) ⇒ Object



589
590
591
592
593
594
595
596
597
598
599
# File 'lib/Dnsruby/Resolver.rb', line 589

def Resolver.get_ports_from(p)
  a = []
  if (p.class == Fixnum)
    a = [p]
  else
    p.each do |x|
      a.push(x)
    end
  end
  return a
end

.get_tsig(args) ⇒ Object



617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
# File 'lib/Dnsruby/Resolver.rb', line 617

def Resolver.get_tsig(args)
  tsig = nil
  if (args.length == 1)
    if (args[0])
      if (args[0].instance_of?RR::TSIG)
        tsig = args[0]
      elsif (args[0].instance_of?Array)
        tsig = RR.new_from_hash({:type => Types.TSIG, :klass => Classes.ANY, :name => args[0][0], :key => args[0][1]})
      end
    else
      #          Dnsruby.log.debug{"TSIG signing switched off"}
      return nil
    end
  elsif (args.length ==2)
    tsig = RR.new_from_hash({:type => Types.TSIG, :klass => Classes.ANY, :name => args[0], :key => args[1]})
  else
    raise ArgumentError.new("Wrong number of arguments to tsig=")
  end
  Dnsruby.log.info{"TSIG signing now using #{tsig.name}, key=#{tsig.key}"}
  return tsig
end

.port_in_range(p) ⇒ Object



581
582
583
584
585
586
587
# File 'lib/Dnsruby/Resolver.rb', line 581

def Resolver.port_in_range(p)
  if ((p == 0) || ((Iana::IANA_PORTS.index(p)) == nil &&
          (p > 1024) && (p < 65535)))
    return true
  end
  return false
end

Instance Method Details

#add_config_nameserversObject

:nodoc: all



393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
# File 'lib/Dnsruby/Resolver.rb', line 393

def add_config_nameservers # :nodoc: all
  if (!@configured)
    @config.get_ready
  end
  @configured = true
  @single_res_mutex.synchronize {
  # Add the Config nameservers
  @config.nameserver.each do |ns|
    @single_resolvers.push(PacketSender.new({:server=>ns, :dnssec=>@dnssec,
          :use_tcp=>@use_tcp, :packet_timeout=>@packet_timeout,
          :tsig => @tsig, :ignore_truncation=>@ignore_truncation,
          :src_address=>@src_address, :src_port=>@src_port,
          :recurse=>@recurse, :udp_size=>@udp_size}))
  end
  }
end

#add_server(server) ⇒ Object

# Add a new SingleResolver to the list of resolvers this Resolver object will

# query.
def add_resolver(internal) # :nodoc:
  # @TODO@ Make a new PacketSender from this SingleResolver!!
  @single_resolvers.push(internal)
end


467
468
469
470
471
472
473
474
# File 'lib/Dnsruby/Resolver.rb', line 467

def add_server(server)# :nodoc:
  @configured = true
  res = PacketSender.new(server)
  update_internal_res(res)
  @single_res_mutex.synchronize {
    @single_resolvers.push(res)
  }
end

#add_src_port(p) ⇒ Object

Can be a single Fixnum or a Range or an Array If an invalid port is selected (one reserved by IANA), then an ArgumentError will be raised. “0” means “any valid port” - this is only a viable option if it is the only port in the list. An ArgumentError will be raised if “0” is added to an existing set of source ports.

res.add_src_port(60000)
res.add_src_port([60001,60005,60010])
res.add_src_port(60015..60115)


546
547
548
549
550
551
552
553
554
555
556
557
# File 'lib/Dnsruby/Resolver.rb', line 546

def add_src_port(p)
  if (Resolver.check_port(p, @src_port))
    a = Resolver.get_ports_from(p)
    a.each do |x|
      if ((@src_port.length > 0) && (x == 0))
        raise ArgumentError.new("src_port of 0 only allowed as only src_port value (currently #{@src_port.length} values")
      end
      @src_port.push(x)
    end
  end
  update
end

#closeObject

Close the Resolver. Unfinished queries are terminated with OtherResolvError.



327
328
329
# File 'lib/Dnsruby/Resolver.rb', line 327

def close
  @resolver_ruby.close if @resolver_ruby
end

#generate_timeouts(base = 0) ⇒ Object

:nodoc: all



690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
# File 'lib/Dnsruby/Resolver.rb', line 690

def generate_timeouts(base=0) #:nodoc: all
  #These should be be pegged to the single_resolver they are targetting :
  #  e.g. timeouts[timeout1]=nameserver
  timeouts = {}
  retry_delay = @retry_delay
#      @single_res_mutex.synchronize {
    @retry_times.times do |retry_count|
      if (retry_count>0)
        retry_delay *= 2
      end
#          servers=[]
#          @single_resolvers.each do |r| servers.push(r.server) end
      @single_resolvers.each_index do |i|
        res= @single_resolvers[i]
        next if !res # @TODO@ WHY?1
        offset = (i*@retry_delay.to_f/@single_resolvers.length)
        if (retry_count==0)
          timeouts[base+offset]=[res, retry_count]
        else
          if (timeouts.has_key?(base+retry_delay+offset))
            Dnsruby.log.error{"Duplicate timeout key!"}
            raise RuntimeError.new("Duplicate timeout key!")
          end
          timeouts[base+retry_delay+offset]=[res, retry_count]
        end
      end
    end
#      }
  return timeouts      
end

#nameserver=(n) ⇒ Object



488
489
490
491
492
493
494
495
# File 'lib/Dnsruby/Resolver.rb', line 488

def nameserver=(n)
  @configured = true
        @single_res_mutex.synchronize {
  @single_resolvers=[]
        }
  set_config_nameserver(n)
  add_config_nameservers
end

#nameservers=(ns) ⇒ Object



485
486
487
# File 'lib/Dnsruby/Resolver.rb', line 485

def nameservers=(ns)
  self.nameserver=(n)
end

#persistent_tcp=(on) ⇒ Object



655
656
657
658
# File 'lib/Dnsruby/Resolver.rb', line 655

def persistent_tcp=(on)
  @persistent_tcp = on
  update
end

#persistent_udp=(on) ⇒ Object



660
661
662
663
# File 'lib/Dnsruby/Resolver.rb', line 660

def persistent_udp=(on)
  @persistent_udp = on
  update
end

#query(name, type = Types.A, klass = Classes.IN, set_cd = @dnssec) ⇒ Object

Query for a name. If a valid Message is received, then it is returned to the caller. Otherwise an exception (a Dnsruby::ResolvError or Dnsruby::ResolvTimeout) is raised.

require 'Dnsruby'
res = Dnsruby::Resolver.new
response = res.query("example.com") # defaults to Types.A, Classes.IN
response = res.query("example.com", Types.MX)
response = res.query("208.77.188.166") # IPv4 address so PTR query will be made
response = res.query("208.77.188.166", Types.PTR)


157
158
159
160
161
162
163
164
165
166
# File 'lib/Dnsruby/Resolver.rb', line 157

def query(name, type=Types.A, klass=Classes.IN, set_cd=@dnssec)
  msg = Message.new
  msg.header.rd = 1
  msg.add_question(name, type, klass)
  msg.do_validation = @do_validation
  if (@dnssec)
    msg.header.cd = set_cd # We do our own validation by default
  end
  return send_message(msg)
end

#query_no_validation_or_recursion(name, type = Types.A, klass = Classes.IN) ⇒ Object

:nodoc: all



168
169
170
171
172
173
174
175
176
177
# File 'lib/Dnsruby/Resolver.rb', line 168

def query_no_validation_or_recursion(name, type=Types.A, klass=Classes.IN) # :nodoc: all
  msg = Message.new
  msg.header.rd = false
  msg.do_validation = false
  msg.add_question(name, type, klass)
  if (@dnssec)
    msg.header.cd = true # We do our own validation by default
  end
  return send_message(msg)
end

#reset_attributesObject

:nodoc: all



424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
# File 'lib/Dnsruby/Resolver.rb', line 424

def reset_attributes #:nodoc: all
  if (@resolver_ruby)
    @resolver_ruby.reset_attributes
  end
 
  # Attributes
  @do_validation = true
  @query_timeout = DefaultQueryTimeout
  @retry_delay = DefaultRetryDelay
  @retry_times = DefaultRetryTimes
  @packet_timeout = DefaultPacketTimeout
  @port = DefaultPort
  @udp_size = DefaultUDPSize
  @dnssec = DefaultDnssec
  @use_tcp = false
  @tsig = nil
  @ignore_truncation = false
  @config = Config.new()
  @src_address        = '0.0.0.0'
  @src_port        = [0]
  @recurse = true
  @single_res_mutex.synchronize {
  @single_resolvers=[]
  }
  @configured = false
end

#send_async(*args) ⇒ Object

Asynchronously send a Message to the server. The send can be done using just Dnsruby. Support for EventMachine has been deprecated.

Dnsruby pure Ruby event loop :

A client_queue is supplied by the client, along with an optional client_query_id to identify the response. The client_query_id is generated, if not supplied, and returned to the client. When the response is known, a tuple of (query_id, response_message, exception) will be added to the client_queue.

The query is sent synchronously in the caller’s thread. The select thread is then used to listen for and process the response (up to pushing it to the client_queue). The client thread is then used to retrieve the response and deal with it.

Takes :

  • msg - the message to send

  • client_queue - a Queue to push the response to, when it arrives

  • client_query_id - an optional ID to identify the query to the client

  • use_tcp - whether to use TCP (defaults to SingleResolver.use_tcp)

Returns :

  • client_query_id - to identify the query response to the client. This ID is

generated if it is not passed in by the client

Example invocations :

id = res.send_async(msg, queue)
NOT SUPPORTED : id = res.send_async(msg, queue, use_tcp)
id = res.send_async(msg, queue, id)
id = res.send_async(msg, queue, id, use_tcp)

Example code :

require 'Dnsruby'
res = Dnsruby::Resolver.newsend
query_id = 10 # can be any object you like
query_queue = Queue.new
res.send_async(Message.new("example.com", Types.MX),  query_queue, query_id)
query_id_2 = res.send_async(Message.new("example.com", Types.A), query_queue)
# ...do a load of other stuff here...
2.times do 
  response_id, response, exception = query_queue.pop
  # You can check the ID to see which query has been answered
  if (exception == nil)
      # deal with good response
  else
      # deal with problem
  end
end


314
315
316
317
318
319
320
321
322
323
324
# File 'lib/Dnsruby/Resolver.rb', line 314

def send_async(*args) # msg, client_queue, client_query_id)
  if (!@configured)
    add_config_nameservers
  end
#      @single_res_mutex.synchronize {
  if (!@resolver_ruby) # @TODO@ Synchronize this?
    @resolver_ruby = ResolverRuby.new(self)
  end
#      }
  return @resolver_ruby.send_async(*args)
end

#send_message(message) ⇒ Object

Send a message, and wait for the response. If a valid Message is received, then it is returned to the caller. Otherwise an exception (a Dnsruby::ResolvError or Dnsruby::ResolvTimeout) is raised.

send_async is called internally.

example :

require 'dnsruby'
include Dnsruby
res = Dnsruby::Resolver.new
begin
response = res.send_message(Message.new("example.com", Types.MX))
rescue ResolvError
  # ...
rescue ResolvTimeout
  # ...
end


196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
# File 'lib/Dnsruby/Resolver.rb', line 196

def send_message(message)
  Dnsruby.log.debug{"Resolver : sending message"}
  q = Queue.new
  send_async(message, q)
  #      # @TODO@ Add new queue tuples, e.g. :
  #      event_type = EventType::RECEIVED
  #      reply = nil
  #      while (event_type == EventType::RECEIVED)
  #        id, event_type, reply, error = q.pop
  #        Dnsruby.log.debug{"Resolver : result received"}
  #        if ((error != nil) && (event_type == EventType::ERROR))
  #          raise error
  #        end
  #        print "Reply = #{reply}\n"
  #      end
  #      print "Reply = #{reply}\n"
  #      return reply

  id, result, error = q.pop

  if (error != nil)
    raise error
  else
    return result
  end
end

#send_plain_message(message) ⇒ Object

This method takes a Message (supplied by the client), and sends it to the configured nameservers. No changes are made to the Message before it is sent (TSIG signatures will be applied if configured on the Resolver). Retries are handled as the Resolver is configured to do. Incoming responses to the query are not cached or validated (although TCP fallback will be performed if the TC bit is set and the (Single)Resolver has ignore_truncation set to false). Note that the Message is left untouched - this means that no OPT records are added, even if the UDP transport for the server is specified at more than 512 bytes. If it is desired to use EDNS for this packet, then you should call the Dnsruby::PacketSender#prepare_for_dnssec(msg), or Dnsruby::PacketSender#add_opt_rr(msg) The return value from this method is the [response, error] tuple. Either of these values may be nil - it is up to the client to check.

example :

require 'dnsruby'
include Dnsruby
res = Dnsruby::Resolver.new
response, error = res.send_plain_message(Message.new("example.com", Types.MX))
if (error)
  print "Error returned : #{error}\n"
else
  process_response(response)
end


249
250
251
252
253
254
255
256
257
258
# File 'lib/Dnsruby/Resolver.rb', line 249

def send_plain_message(message)
  Dnsruby::TheLog.debug("Resolver : send_plain_message")
  message.do_caching = false
  message.do_validation = false
  message.send_raw = true
  q = Queue.new
  send_async(message, q)
  id, result, error = q.pop
  return [result, error]
end

#set_config_nameserver(n) ⇒ Object



410
411
412
413
414
415
416
417
418
419
420
421
422
# File 'lib/Dnsruby/Resolver.rb', line 410

def set_config_nameserver(n)
  # @TODO@ Should we allow NS RRSet here? If so, then .sort_by {rand}
  if (!@configured)
    @config.get_ready
  end
  @configured = true
  if (n).kind_of?String
    @config.nameserver=[n]
  else
    @config.nameserver=n
  end
  add_config_nameservers
end

#single_res_mutexObject

:nodoc: all



686
687
688
# File 'lib/Dnsruby/Resolver.rb', line 686

def single_res_mutex # :nodoc: all
  return @single_res_mutex
end

#single_resolversObject

}



110
111
112
113
114
115
# File 'lib/Dnsruby/Resolver.rb', line 110

def single_resolvers # :nodoc:
  if (!@configured)
    add_config_nameservers
  end
  return @single_resolvers
end

#single_resolvers=(s) ⇒ Object

The array of SingleResolvers used for sending query messages

attr_accessor :single_resolvers # :nodoc:


104
105
106
107
108
109
# File 'lib/Dnsruby/Resolver.rb', line 104

def single_resolvers=(s) # :nodoc:
  @configured = true
#      @single_res_mutex.synchronize {
  @single_resolvers = s
#      }
end

#src_portObject

The source port to send queries from Returns either a single Fixnum or an Array e.g. “0”, or “[60001, 60002, 60007]”

Defaults to 0 - random port



512
513
514
515
516
517
# File 'lib/Dnsruby/Resolver.rb', line 512

def src_port
  if (@src_port.length == 1) 
    return @src_port[0]
  end
  return @src_port
end

#src_port=(p) ⇒ Object

Can be a single Fixnum or a Range or an Array If an invalid port is selected (one reserved by IANA), then an ArgumentError will be raised.

res.src_port=0
res.src_port=[60001,60005,60010]
res.src_port=60015..60115


527
528
529
530
531
532
# File 'lib/Dnsruby/Resolver.rb', line 527

def src_port=(p)
  if (Resolver.check_port(p))
    @src_port = Resolver.get_ports_from(p)
    update
  end
end

#updateObject

:nodoc: all



451
452
453
454
455
456
457
458
# File 'lib/Dnsruby/Resolver.rb', line 451

def update #:nodoc: all
  #Update any resolvers we have with the latest config
  @single_res_mutex.synchronize {
    @single_resolvers.each do |res|
      update_internal_res(res)
    end
  }
end

#update_internal_res(res) ⇒ Object



476
477
478
479
480
481
482
483
# File 'lib/Dnsruby/Resolver.rb', line 476

def update_internal_res(res)
  [:port, :use_tcp, :tsig, :ignore_truncation, :packet_timeout,
    :src_address, :src_port, :recurse,
    :udp_size, :dnssec].each do |param|

    res.send(param.to_s+"=", instance_variable_get("@"+param.to_s))
  end
end